#region Copyright Syncfusion Inc. 2001 - 2005
//
//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid.Grouping;


namespace GroupingPerf
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Panel gridPanel;
		private GridGroupingControl grid;

//		private System.Windows.Forms.LinkLabel lblShow;
		private DataTable table;

		int randomKey = 25;
		int startId = 1000;
		int recordCount = 10000;
		int endId = 100000;
		bool calculateMaxColumnWidth = false;
		bool useGroupingSortList = false;
		bool sortandCategorize = true;
//		bool showAll = true;
		private System.Windows.Forms.LinkLabel lblShow;

		string[] states = new string[]{"North Carolina", "South Carolina", "Washington", "Nevada", "Ohio"};

		public Form1()
		{
			// GroupingEngineFactory provides a trimmed down GridGroup
			// which eliminates overhead of not needed preview rows, header and footer cells.
			GridEngineFactory.Factory = new GroupingEngineFactory();

			InitializeComponent();

			InitializeDataTable();

			InitializeGroupingGrid();

			this.grid.TableDescriptor.Columns["id"].AllowFilter = true;
			this.grid.QueryCellStyleInfo += new GridTableCellStyleInfoEventHandler(grid_QueryCellStyleInfo);
			
		}



		private void InitializeDataTable()
		{
			Random random = new Random(randomKey++);
			int numStates = states.GetLength(0);
			string nameFormat = "Name{0:0000}";

			this.table = new DataTable("Test");

			DataColumn col = table.Columns.Add();
			col.DataType = typeof(string);
			col.ColumnName = "Name";

			col = table.Columns.Add();
			col.DataType = typeof(string);
			col.ColumnName = "State";

			col = table.Columns.Add();
			col.DataType = typeof(int);
			col.ColumnName = "id";

			for(int i = 0; i < recordCount; i++)
			{
				DataRow row = this.table.NewRow();
				row["Name"] = string.Format(nameFormat, i/100);
				row["State"] = states[random.Next(numStates-1)];

				// To gave negative id for Nevada 
				
				if (row["State"].ToString() == "Nevada" && row["Name"].ToString() == "Name0007" ||
					row["State"].ToString() == "South Carolina")
				{
					row["id"] = random.Next(endId * -1,startId * -1);	
				}
				else
				{
					row["id"] = random.Next(startId,endId);
				}
				this.table.Rows.Add(row);
			}
		}

		private void InitializeGroupingGrid()
		{
			if (this.grid != null)
				this.grid.Dispose();


			// Fill DataTable
			InitializeDataTable();


			this.grid = new GridGroupingControl();
			this.grid.TableDescriptor.AllowCalculateMaxColumnWidth = calculateMaxColumnWidth;
			this.grid.Dock = DockStyle.Fill;

			this.grid.Engine.AllowSwapDataViewWithDataTableList = true;

			this.grid.SourceListListChanged += new TableListChangedEventHandler(grid_SourceListListChanged);
//			this.lblShow.Click += new EventHandler(lblShow_Click);
			// Assign data source
			if (useGroupingSortList)
			{
				GroupingSortList gs = new GroupingSortList(this.table);
				this.grid.DataSource = gs;
			}
			else
				this.grid.DataSource = this.table;

			// Assign grouping and sort order

//			SortColumnDescriptor sortCol = new Syncfusion.Grouping.SortColumnDescriptor();
//			sortCol.Comparer
			if (this.sortandCategorize)
			{
				this.grid.TableDescriptor.GroupedColumns.Add("State");
				this.grid.TableDescriptor.GroupedColumns.Add("Name");
			}

			// Force filling and intialization of engine and drawing of grid.
//			int count = this.grid.Table.DisplayElements.Count;
//

			// paint the grid.
			this.gridPanel.Controls.Add(this.grid);
			this.grid.Update();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.gridPanel = new System.Windows.Forms.Panel();
			this.lblShow = new System.Windows.Forms.LinkLabel();
			this.SuspendLayout();
			// 
			// gridPanel
			// 
			this.gridPanel.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.gridPanel.Location = new System.Drawing.Point(80, 72);
			this.gridPanel.Name = "gridPanel";
			this.gridPanel.Size = new System.Drawing.Size(464, 288);
			this.gridPanel.TabIndex = 0;
			// 
			// lblShow
			// 
			this.lblShow.Location = new System.Drawing.Point(88, 48);
			this.lblShow.Name = "lblShow";
			this.lblShow.Size = new System.Drawing.Size(88, 23);
			this.lblShow.TabIndex = 1;
			this.lblShow.TabStop = true;
			this.lblShow.Text = "Hide Negative";
			this.lblShow.LinkClicked += new System.Windows.Forms.LinkLabelLinkClickedEventHandler(this.lblShow_LinkClicked);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(664, 566);
			this.Controls.Add(this.lblShow);
			this.Controls.Add(this.gridPanel);
			this.MinimumSize = new System.Drawing.Size(400, 600);
			this.Name = "Form1";
			this.Text = "Grouping Performance";
			this.ResumeLayout(false);

		}
		#endregion


		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.Run(new Form1());
		}


		private void grid_SourceListListChanged(object sender, TableListChangedEventArgs e)
		{
			// use optimization. Grid is not filtered by Name and we also don't use custom counters.
			// So, as long as the sort order is not modified, there is no need to to invalidate
			// counters bottom up
			e.ShouldInvalidateSummaries = false;
			e.ShouldInvalidateCounters = false;
		}

		ArrayList extraSections = new ArrayList();
		private void lblShow_LinkClicked(object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e)
		{
			
			if (this.lblShow.Text == "Show All")
			{
				this.grid.TableDescriptor.RecordFilters.Clear();
				this.lblShow.Text = "Hide Negative";							
			}
			else
			{
				this.lblShow.Text = "Show All";
				this.grid.TableDescriptor.RecordFilters.Add("[id] > 0");
			}
		}

		private void grid_QueryCellStyleInfo(object sender, GridTableCellStyleInfoEventArgs e)
		{
			if (e.TableCellIdentity.DisplayElement is CaptionRow)
			{
				Element el = e.TableCellIdentity.DisplayElement;
				if(el.ParentGroup.GetFilteredRecordCount() == 0 && el.ParentGroup.Groups.Count == 0 &&
					e.TableCellIdentity.TableCellType == GridTableCellType.GroupCaptionPlusMinusCell)
				{
					el.ParentGroup.IsExpanded = false;
					e.Style.CellType = "Header";
				}

				if(e.TableCellIdentity.TableCellType == GridTableCellType.GroupCaptionCell)
					e.Style.CellValue  = String.Format("{0}: {1} - {2} Items", el.ParentGroup.Name,el.ParentGroup.Category, el.ParentGroup.GetRecordCount());
			}
		}

	}

	/// <summary>
	/// GroupingSortList wraps a DataView with a IBindingList and implements the IGroupingList
	/// interface and its Sort method. This allows performing the sort on the dataview directly
	/// instead of letting the grouping engine perform the sorting.
	/// </summary>
	public class GroupingSortList : IGroupingList, IBindingList
	{
		DataTable dt;
		DataView dv;
		IBindingList bindingList;

		public GroupingSortList(DataTable dt)
		{
			this.dt = dt;
			this.dv = dt.DefaultView;
			bindingList = (IBindingList) dv;
			bindingList.ListChanged += new ListChangedEventHandler(bindingList_ListChanged);
		}

		#region IGroupingList Members

		public void ApplySort(RelationChildColumnDescriptorCollection relationChildColumns, SortColumnDescriptorCollection groupColumns, SortColumnDescriptorCollection sortColumns)
		{
			bool first = true;
			StringBuilder sb = new StringBuilder();
			foreach (SortColumnDescriptor sd in relationChildColumns)
			{
				if (first)
					first = false;
				else
					sb.Append(", ");

				sb.Append(sd.FieldDescriptor.MappingName);
				if (sd.SortDirection == ListSortDirection.Descending)
					sb.Append(" DESC");
			}
			foreach (SortColumnDescriptor sd in groupColumns)
			{
				if (first)
					first = false;
				else
					sb.Append(", ");

				sb.Append(sd.FieldDescriptor.MappingName);
				if (sd.SortDirection == ListSortDirection.Descending)
					sb.Append(" DESC");
			}
			foreach (SortColumnDescriptor sd in sortColumns)
			{
				if (first)
					first = false;
				else
					sb.Append(", ");

				sb.Append(sd.FieldDescriptor.MappingName);
				if (sd.SortDirection == ListSortDirection.Descending)
					sb.Append(" DESC");
			}

			string sort = sb.ToString();
			if (sort != dv.Sort)
			{
				dv.Sort = sort;
			}
		}

		public bool AllowItemReference
		{
			get
			{
				return false;
			}
		}

		public bool SupportsGroupSorting
		{
			get
			{
				return true;
			}
		}

		public Syncfusion.Grouping.GroupingSortBehavior GroupingSortBehavior
		{
			get
			{
				return Syncfusion.Grouping.GroupingSortBehavior.GroupByGroup;
			}
		}

		#endregion

		#region IBindingList Members

		public void AddIndex(PropertyDescriptor property)
		{
			bindingList.AddIndex(property);
		}

		public bool AllowNew
		{
			get
			{
				return bindingList.AllowNew;
			}
		}

		void System.ComponentModel.IBindingList.ApplySort(PropertyDescriptor property, System.ComponentModel.ListSortDirection direction)
		{
			bindingList.ApplySort(property, direction);
		}

		public PropertyDescriptor SortProperty
		{
			get
			{
				return bindingList.SortProperty;
			}
		}

		public int Find(PropertyDescriptor property, object key)
		{
			return bindingList.Find(property, key);
		}

		public bool SupportsSorting
		{
			get
			{
				return bindingList.SupportsSorting;
			}
		}

		public bool IsSorted
		{
			get
			{
				return bindingList.IsSorted;
			}
		}

		public bool AllowRemove
		{
			get
			{
				return bindingList.AllowRemove;
			}
		}

		public bool SupportsSearching
		{
			get
			{
				return bindingList.SupportsSearching;
			}
		}

		public System.ComponentModel.ListSortDirection SortDirection
		{
			get
			{
				return bindingList.SortDirection;
			}
		}

		public event System.ComponentModel.ListChangedEventHandler ListChanged;

		public bool SupportsChangeNotification
		{
			get
			{
				return bindingList.SupportsChangeNotification;
			}
		}

		public void RemoveSort()
		{
			bindingList.RemoveSort();
		}

		public object AddNew()
		{
			return bindingList.AddNew();
		}

		public bool AllowEdit
		{
			get
			{
				return bindingList.AllowEdit;
			}
		}

		public void RemoveIndex(PropertyDescriptor property)
		{
			bindingList.RemoveIndex(property);
		}

		#endregion

		#region IList Members

		public bool IsReadOnly
		{
			get
			{
				return bindingList.IsReadOnly;
			}
		}

		public object this[int index]
		{
			get
			{
				return bindingList[index];
			}
			set
			{
				bindingList[index] = value;
			}
		}

		public void RemoveAt(int index)
		{
			bindingList.RemoveAt(index);
		}

		public void Insert(int index, object value)
		{
			bindingList.Insert(index, value);
		}

		public void Remove(object value)
		{
			bindingList.Remove(value);
		}

		public bool Contains(object value)
		{

			return bindingList.Contains(value);
		}

		public void Clear()
		{
			bindingList.Clear();
		}

		public int IndexOf(object value)
		{
			return bindingList.IndexOf(value);
		}

		public int Add(object value)
		{
			return bindingList.Add(value);
		}

		public bool IsFixedSize
		{
			get
			{
				return bindingList.IsFixedSize;
			}
		}

		#endregion

		#region ICollection Members

		public bool IsSynchronized
		{
			get
			{
				return bindingList.IsSynchronized;
			}
		}

		public int Count
		{
			get
			{
				return bindingList.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			bindingList.CopyTo(array, index);
		}

		public object SyncRoot
		{
			get
			{
				return bindingList.SyncRoot;
			}
		}

		#endregion

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return bindingList.GetEnumerator();
		}

		#endregion

		private void bindingList_ListChanged(object sender, ListChangedEventArgs e)
		{
			if (ListChanged != null)
				ListChanged(this, e);
		}
	}


}
